==> imwiki-0.6.4.tar.gz: imwiki-0.6.4/imLib/webInterface.py <==
# -*- coding: utf8 -*-
import time
import os
import Cookie 

from utils import *
import Category
import Page


from imParser import WikiParser
from imFormatter import HTMLFormatter
import User
import UserDB
import Session
import TagManager
import Tag
import Revision
import Logger
import Request
import Response
import templateProcessor
import config
from version import *

from WikiException import WikiException
class NotExistActionError(WikiException): pass 
class InvalidRawText(WikiException): pass
class CheckYourPassword(WikiException): pass
class CheckYourEmail(WikiException): pass
class CheckYourId(WikiException): pass
class InvalidUserId(WikiException): pass

def getUserAction(actionName):
	try:
		module = __import__('imLib.imAction.' + actionName, globals(), locals(), True)
	except ImportError:
		return None
	return module.__dict__.get('do_'+actionName)

def main():
	try:
		request = Request.Request()
		request.sess = Session.Session()
		if not request.sess.has_key('uid'):
			request.sess['uid'] = 'anonymous'
		request.user = User.User(request.sess['uid'])
		templateProcessor.setTemplateFolder(config.templateFolderPath) 
		actionFunc = globals().get('do_' + request.action)
		if not actionFunc:
			actionFunc = getUserAction(request.action)
		if not actionFunc:
			raise NotExistActionError
		response = actionFunc(request)
	except IOError:
		print 'Content-Type: text/html\n'
		print 'Unknown Exception'
	#except WikiException, e:
		#e.printMessage()
	#j	print 'Content-Type: text/plain\n'
	#	print e.__class__
		#print e.args[0].keys()
	#except:
#		print 'Content-Type: text/html\n'
#		print 'Unknown Exception'
	else:
		print response

def redirect(url):
	print 'Location: %s' % url
	# for test vvvvvvvvvvvvvvvvvvvvv
	print 'Content-Type: text/plain\n'
	return 'Redirect: %s' % url 

def warning(title, warningMsg):
	request = Request.Request()
	return Response.Response('warning.teul', globals(), locals())

def do_view(request):
	page = Page.travelAlongRails(request.categoryPath, request.pageName)
	
	if page.category.categoryPath != config.globalCategoryName and page.category.categoryPath != request.categoryPath:
		redirectPath = joinPath(request.scriptName, encode(page.category.categoryPath), page.encPageName)
		return redirect(redirectPath)
	if not page: 
		templateCategory = Category.getCategoryByCategoryPath('Template')
		return Response.Response('noPage.teul', globals(), locals())
	if not page.permCheck(request.user, 'read'): 
		return warning('Permission Error', "You don't have permission.")
	parser = WikiParser.Parser(HTMLFormatter.HTMLFormatter())
	parser.feed(page.rawText)
	parsedText = parser.parse()
	tagLinks = []
	for tag in TagManager.TagManager(page.tags).toSet():
		tagLinks.append(Tag.Tag(tag).toLink())
	tagLinks = ', '.join(tagLinks)
	title = page.pageName
	return Response.Response('view.teul', globals(), locals())

def do_raw(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	rev = Revision.Revision(category, request.pageName, request.timeStamp) # FIXME NotExistRevision
	if not rev.permCheck(request.user, 'read'): # FIXME permCheck
		return warning('Permission Error', "You don't have permission.")
	print 'Content-Type: text/plain; charset=%s\n' % config.charset
	return rev.toRawText()

def do_diff(request): 
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		#raise NotExistCategory(do_view)
		#raise NotExistCategory(action, title, msg, request)
		return do_view(request)
	page = category.getPage(request.pageName) 
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not page.permCheck(request.user, 'read'): 
		return warning('Permission Error', "You don't have permission.")
	if not request.timeStamp:
		request.timeStamp = Revision.getLastTimeStamp(request.categoryPath, request.pageName)
	if not request.timeStamp:
		request.msg = 'No Revision'
		return do_view(request)
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	rev = Revision.Revision(category, request.pageName, request.timeStamp)
	ret = diff(rev.rawText, page.rawText) # FIXME 기능을 추가하게되면 모듈로 분리할 것. 그때까지 임시로 utils에 두자.
	ret = nl2br(ret) # FIXME 이거랑..< 이런것들  엔터티로 처리
	# FIXME title = 'Diff : pageName'
	return Response.Response('diff.teul', globals(), locals())

def do_writeForm(request):
	category = Category.getCategoryByCategoryPath(joinPath(request.categoryPath, request.pageName)) # pageName이 생략된 경우
	if category: 
		# FIXME vvvvvvvvvvvvvv 더러운 코드
		request.categoryPath = joinPath(request.categoryPath, request.pageName)
		request.pageName = 'FrontPage'
		request.setActionPath(request.categoryPath, request.pageName)
	tags = ''
	rawText = ''
	if request.template: # FIXME FIXME FIXME FIXME
		templateCategory = Category.getCategoryByCategoryPath(config.templateCategoryName)
		if templateCategory:
			templatePage = templateCategory.getPage(request.template)
			if templatePage:
				tags = templatePage.tags
				rawText = templatePage.rawText
	category = Category.getCategoryByCategoryPath(request.categoryPath, exist=True)
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	memberRead, memberWrite, groupRead, groupWrite, anonyRead, anonyWrite = category.perms.toBoolList() # skin에서 사용
	return Response.Response('write.teul', globals(), locals())

def do_write(request):
	rawText = request.rawText
	tags = request.tags
	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))
	# FIXME 1 카테고리의 권한이 없을 때 2,3카테고리를 만들어 페이지를 만들 수 있는게 아닌가.
	if not rawText.strip():
		request.msg = 'Empty text'
		return do_editForm(request)
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		category = Category.createCategoryByCategoryPath(request.categoryPath, perms)
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	if category.getPage(request.pageName):
		request.msg = 'Exist page'
		return do_view(request)
	Page.createPage(category, request.pageName, rawText, tags, perms)
	Logger.editLog('do_write', request.categoryPath, request.pageName, time.time(), request.user.uid)
	return redirect(request.actionPath)



def do_editForm(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		return do_view(request)
	page = category.getPage(request.pageName)
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not (page.permCheck(request.user, 'read') and page.permCheck(request.user, 'write')): 
		return warning('Permission Error', "You don't have permission.")
	memberRead, memberWrite, groupRead, groupWrite, anonyRead, anonyWrite = page.perms.toBoolList() # skin에서 사용
	return Response.Response('edit.teul', globals(), locals())

def do_edit(request):
	rawText = request.rawText
	tags = request.tags
	mtime = request.mtime
	if not rawText.strip():
		request.msg = 'Empty text'
		return do_editForm(request)
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		return do_view(request)
	page = category.getPage(request.pageName)
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not page.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	if mtime != str(page.mtime):
		request.msg = 'Edit conflict'
		return do_editForm(request)

	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))

	if rawText == page.rawText and tags == page.tags and perms == page.perms:
		request.msg = 'No change, No Save'
		return do_view(request)
	page.backup()

	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))
	page.edit(rawText, tags, perms, mtime) # FIXME mtime 사용되지 않음.
	Logger.editLog('do_edit', request.categoryPath, request.pageName, time.time(), request.user.uid)
	return redirect(request.actionPath)


def do_remove(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		return do_view(request)
	page = category.getPage(request.pageName)
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not page.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	if not request.timeStamp:
		request.msg = '''<form method="POST" action="%s">
	<strong>Are you sure?</strong>
	<input type="hidden" name="action" value="remove">
	<input type="hidden" name="timeStamp" value="%s">
	<input type="submit" name="button" value="Remove">
</form>''' % (request.actionPath, page.mtime)
		return do_view(request)
	if request.timeStamp != str(page.mtime):
		request.msg = 'Invalid timeStamp'
		return do_view(request)
	page.backup()
	page.remove()
	Logger.editLog('do_remove', request.categoryPath, request.pageName, time.time(), request.user.uid)

	redirectPath = joinPath(request.scriptName, request.categoryPath, 'RecentChanges')
	return redirect(redirectPath)

def do_titleSearch(request):
	if not request.needle.strip():
		request.msg = 'Empty needle'
		return do_view(request)
	title = "Title Search : %s" % request.needle
	page = Page.travelAlongRails(request.categoryPath, request.needle)
	if page:
		redirectPath = joinPath(request.scriptName, page.category.categoryPath, page.pageName)
		return redirect(redirectPath)
	# FIXME pageList 자식 카테고리에 여러개의 페이지가 있을경우의 pageList
	# 단어를 페이지 이름에 포함하는 pageList를 
	# 만들어줘야함.
	templateCategory = Category.getCategoryByCategoryPath('Template')
	return Response.Response('titleSearch.teul', globals(), locals())

def do_fullSearch(request):
	title = "Full Search : %s" % request.needle
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		return do_view(request)
	pageList = category.getPageListFromDescendants(lambda page: request.needle in page.rawText)
	return Response.Response('fullSearch.teul', globals(), locals())

def do_join(request):
	# FIXME
	# uid, passwd, email에 사용할 수 없는 문자가 들었나 검사
	# uid : 한글, 영어, 숫자
	# passwd : [a-zA-Z0-9]~!@#$%^&*()-+_= 
	# email은 email룰 그대로
	uid = request.uid.strip()
	passwd = request.passwd.strip()
	passwdConfirm = request.passwdConfirm.strip()
	email = request.email.strip()
	# uid 검사
	if len(uid) < 5:
		request.msg = 'Check your ID (too short)' 
		return do_view(request)
	invalidUserId = ['anonymous']
	if uid.lower() in invalidUserId:
		request.msg = 'Check your ID (invalid id)'
		return do_view(request)
	# passwd 검사
	if len(passwd) < 6 or passwd != passwdConfirm:
		request.msg = 'Check your password (password != password confirm)'
		return do_view(request)
	# email검사
	if email.count('@') != 1:
		request.msg = 'Invalid email address'
		return do_view(request)
	userDB = UserDB.UserDB()
	user = userDB.addUser(uid, passwd, email) # AlreadyExistUser
	request.sess['uid'] = user.uid
	return redirect(request.actionPath)

def do_login(request):# FIXME
	userDB = UserDB.UserDB()
	request.uid = request.uid.strip()
	if not request.uid:
		request.msg = 'Check your ID'
		return do_view(request)
	request.passwd = request.passwd.strip()
	if not request.passwd:
		request.msg = 'Check your password'
		return do_view(request)
	try:
		user = userDB.getUser(request.uid, request.passwd) # getUser를 써야하나? 그냥 User에서 가져오면 안되나 User.User(request.uid)
	except User.NotExistUser:
		request.msg = 'Check your ID'
		return do_view(request)
	except UserDB.NotCorrectPassword:
		request.msg = 'Check your password'
		return do_view(request)
	request.sess['uid'] = user.uid
	return redirect(request.actionPath)

def do_logout(request):
	request.sess.remove()
	request.sess['uid'] = 'anonymous'
	request.user = User.User(request.sess['uid']) # FIXME NotExistUser, InvalidUserData
	return redirect(request.actionPath)

def do_updateUser(request):
	passwd = request.passwd.strip()
	passwdConfirm = request.passwdConfirm.strip()
	email = request.email.strip()
	# uid 검사
	invalidUserId = ['anonymous']
	if request.user.uid.lower() in invalidUserId:
		raise InvalidUserId
	# passwd 검사
	if passwd and (len(passwd) < 6 or passwd != passwdConfirm):
		request.msg = 'Check your password'
		return do_view(request)
	# email검사
	if email.count('@') != 1:
		request.msg = 'Check your email'
		return do_view(request)
	userDB = UserDB.UserDB()
	user = User.User(request.user.uid) # FIXME NotExistUser, InvalidUserData
	user.update(passwd, email)
	request.msg = 'Updated'
	return do_view(request)
	#return redirect(request.actionPath)

def do_pageListInTag(request):
	title = "Page list in '%s' tag" % decode(request.tagName)
	tagObj = Tag.Tag(decode(request.tagName)) 
	return Response.Response('pageListInTag.teul', globals(), locals())

def do_info(request):
	timeStampList = Revision.getRevTimeStampList(request.categoryPath, request.pageName)
	if timeStampList:
		category = Category.getCategoryByCategoryPath(request.categoryPath)
		rev = Revision.Revision(category, request.pageName, timeStampList[-1]) # FIXME -1일 필요가 없지 않나. 어차피 어느 리비젼이든 perms는 같은 값이니까.
		if not rev.permCheck(request.user, 'read'): # FIXME permCheck
			return warning('Permission Error', "You don't have permission.")
	timeStampList.reverse()
	if not request.showAll and len(timeStampList) > 10:
		timeStampList = timeStampList[:10]
	return Response.Response('pageInfo.teul', globals(), locals())

def do_editCategoryForm(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	title = 'Category Edit : %s' % (category.categoryPath and category.categoryPath or 'Root')
	memberRead, memberWrite, groupRead, groupWrite, anonyRead, anonyWrite = category.perms.toBoolList() # skin에서 사용
	return Response.Response('editCategory.teul', globals(), locals())

def do_editCategory(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath) # FIXME categoryPath 그대로 사용해도 되나?
	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))

	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	category.editPerms(perms)
	category.editGroup(request.group.split())

	request.msg = 'changed'
	return do_editCategoryForm(request)

==> imwiki-7.5.27.tar.gz: imwiki-7.5.27/imLib/webInterface.py <==
# -*- coding: utf8 -*-
import time
import os
import Cookie 

from utils import *
import Category
import Page


from imParser import WikiParser
from imFormatter import HTMLFormatter
import User
import UserDB
import Session
import TagManager
import Tag
import Revision
import Logger
import Request
import Response
import templateProcessor
import config
from version import *

from WikiException import WikiException
class NotExistActionError(WikiException): pass 
class InvalidRawText(WikiException): pass
class CheckYourPassword(WikiException): pass
class CheckYourEmail(WikiException): pass
class CheckYourId(WikiException): pass
class InvalidUserId(WikiException): pass

def getUserAction(actionName):
	try:
		module = __import__('imLib.imAction.' + actionName, globals(), locals(), True)
	except ImportError:
		return None
	return module.__dict__.get('do_'+actionName)

def main():
	request = Request.Request()
	request.sess = Session.Session()
	if not request.sess.has_key('uid'):
		request.sess['uid'] = 'anonymous'
	request.user = User.User(request.sess['uid'])
	templateProcessor.setTemplateFolder(config.templateFolderPath) 
	actionFunc = globals().get('do_' + request.action)
	if not actionFunc:
		actionFunc = getUserAction(request.action)
	if not actionFunc:
		raise NotExistActionError
	response = actionFunc(request)
	print response

def redirect(url):
	print 'Location: %s' % url
	# for test vvvvvvvvvvvvvvvvvvvvv
	print 'Content-Type: text/plain\n'
	return 'Redirect: %s' % url 

def warning(title, warningMsg):
	request = Request.Request()
	return Response.Response('warning.teul', globals(), locals())

def do_view(request): # redirect 안쓰게-_-?
	category  = Category.getCategoryByCategoryPath(joinPath(request.categoryPath, request.pageName))
	if category:
		redirectPath = joinPath(request.scriptName, category.categoryPath, 'FrontPage')
		return redirect(redirectPath)
	if not Category.getCategoryByCategoryPath(request.categoryPath):
		return warning('Not Exist Category', 'Not exist category')

	page = Page.travelAlongRails(request.categoryPath, request.pageName)
	if not page: 
		templateCategory = Category.getCategoryByCategoryPath('Template')
		return Response.Response('noPage.teul', globals(), locals())

	#if page.category.categoryPath != 'Global' and page.category.categoryPath != request.categoryPath:
	if page.category.categoryPath != request.categoryPath and page.category.categoryPath != 'Global':
		redirectPath = joinPath(request.scriptName, page.category.categoryPath, page.pageName)
		return redirect(redirectPath)
	category = Category.getCategoryByCategoryPath(request.categoryPath) # for skin
	if not page.permCheck(request.user, 'read'): 
		return warning('Permission Error', "You don't have permission.")
	parser = WikiParser.Parser(HTMLFormatter.HTMLFormatter())
	parser.feed(page.rawText)
	parsedText = parser.parse()
	tagLinks = []
	for tag in TagManager.TagManager(page.tags).toSet():
		tagLinks.append(Tag.Tag(tag).toLink())
	tagLinks = ', '.join(tagLinks)
	title = page.pageName
	return Response.Response('view.teul', globals(), locals())

def do_raw(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	try:
		rev = Revision.Revision(category, request.pageName, request.timeStamp) 
	except Revision.NotExistRevision:
		return warning('Not Exist Revision', 'Not exist revision.')
	if not rev.permCheck(request.user, 'read'): 
		return warning('Permission Error', "You don't have permission.")
	print 'Content-Type: text/plain; charset=%s\n' % config.charset
	return rev.toRawText()

def do_diff(request): 
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		return do_view(request)
	page = category.getPage(request.pageName) 
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not page.permCheck(request.user, 'read'): 
		return warning('Permission Error', "You don't have permission.")
	if not request.timeStamp:
		request.timeStamp = Revision.getLastTimeStamp(request.categoryPath, request.pageName)
	if not request.timeStamp:
		request.msg = 'No Revision'
		return do_view(request)
	rev = Revision.Revision(category, request.pageName, request.timeStamp)
	ret = rev.diff(page) 
	ret = nl2br(ret) 
	title = 'Diff : %s' % page.pageName
	return Response.Response('diff.teul', globals(), locals())

def do_write(request):
	rawText = request.rawText
	tags = request.tags
	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))
	# FIXME 1 카테고리의 권한이 없을 때 2,3카테고리를 만들어 페이지를 만들 수 있는게 아닌가.
	if not rawText.strip():
		request.msg = 'Empty text'
		return do_writeForm(request)
	category = Category.getCategoryByCategoryPath(request.writeCategoryPath)
	if not category:
		category = Category.createCategoryByCategoryPath(request.writeCategoryPath, perms)
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	if category.getPage(request.pageName):
		request.msg = 'Exist page'
		return do_view(request)
	page = Page.createPage(category, request.pageName, rawText, tags, perms)
	Logger.editLog('do_write', encode(category.categoryPath), page.encPageName, time.time(), request.user.uid)
	redirectPath = joinPath(request.scriptName, category.categoryPath, page.pageName)
	return redirect(redirectPath)

def do_writeForm(request):
	tags = ''
	rawText = ''
	templatePage = Page.getPage(config.templateCategoryName, request.template)
	if templatePage:
		tags = templatePage.tags
		rawText = templatePage.rawText
	category = Category.getCategoryByCategoryPath(request.writeCategoryPath, exist=True) #FIXME 이게 괜찮은가?
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	memberRead, memberWrite, groupRead, groupWrite, anonyRead, anonyWrite = category.perms.toBoolList() # skin에서 사용
	return Response.Response('write.teul', globals(), locals())



def do_editForm(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		return do_view(request)
	page = category.getPage(request.pageName)
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not (page.permCheck(request.user, 'read') and page.permCheck(request.user, 'write')): 
		return warning('Permission Error', "You don't have permission.")
	memberRead, memberWrite, groupRead, groupWrite, anonyRead, anonyWrite = page.perms.toBoolList() # skin에서 사용
	return Response.Response('edit.teul', globals(), locals())

def do_edit(request):
	rawText = request.rawText
	tags = request.tags
	mtime = request.mtime
	if not rawText.strip():
		request.msg = 'Empty text'
		return do_editForm(request)
	page = Page.getPage(request.categoryPath, request.pageName)
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not page.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	if mtime != str(page.mtime):
		request.msg = 'Edit conflict'
		return do_editForm(request)

	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))

	if rawText == page.rawText and tags == page.tags and perms == page.perms:
		request.msg = 'No change, No Save'
		return do_view(request)
	page.backup()
	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))
	page.edit(rawText, tags, perms) 
	Logger.editLog('do_edit', encode(page.category.categoryPath), page.encPageName, time.time(), request.user.uid)
	return redirect(request.actionPath)


def do_remove(request):
	page = Page.getPage(request.categoryPath, request.pageName)
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not page.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	if not request.timeStamp:
		request.msg = '''<form method="POST" action="%s">
	<strong>Are you sure?</strong>
	<input type="hidden" name="action" value="remove">
	<input type="hidden" name="timeStamp" value="%s">
	<input type="submit" name="button" value="Remove">
</form>''' % (request.actionPath, page.mtime)
		return do_view(request)
	if request.timeStamp != str(page.mtime):
		request.msg = 'Invalid timeStamp'
		return do_view(request)
	page.backup()
	page.remove()
	Logger.editLog('do_remove', encode(page.category.categoryPath), page.encPageName, time.time(), request.user.uid)

	if os.path.exists(request.categoryPath):
		categoryPath = request.categoryPath
	else:
		categoryPath = page.category.parent and page.category.parent.categoryPath or ''
	redirectPath = joinPath(request.scriptName, categoryPath, 'RecentChanges')
	return redirect(redirectPath)

def do_titleSearch(request):
	if not request.needle.strip():
		request.msg = 'Empty needle'
		return do_view(request)
	title = "Title Search : %s" % request.needle
	page = Page.travelAlongRails(request.categoryPath, request.needle)
	if page:
		redirectPath = joinPath(request.scriptName, page.category.categoryPath, page.pageName)
		return redirect(redirectPath)
	# FIXME pageList 자식 카테고리에 여러개의 페이지가 있을경우의 pageList
	# 단어를 페이지 이름에 포함하는 pageList를 
	# 만들어줘야함.
	templateCategory = Category.getCategoryByCategoryPath('Template')
	return Response.Response('titleSearch.teul', globals(), locals())

def do_fullSearch(request):
	title = "Full Search : %s" % request.needle
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		return do_view(request)
	pageList = category.getPageListFromDescendants(lambda page: request.needle in page.rawText)
	return Response.Response('fullSearch.teul', globals(), locals())

def do_join(request):
	uid = request.uid.strip()
	passwd = request.passwd.strip()
	passwdConfirm = request.passwdConfirm.strip()
	email = request.email.strip()
	if not isValidName(uid, minLength=5, maxLength=16, whitespaceCheck=True) or uid.lower() == 'anonymous':
		request.msg = str(d)
		return do_view(request)
	if not (5 < len(passwd) < 17) or passwd != passwdConfirm:
		request.msg = 'Check your password (5 < 비밀번호 길이 < 17, password, passwordConfirm 이어야 합니다.)'
		return do_view(request)
	if email.count('@') != 1:
		request.msg = 'Invalid email address'
		return do_view(request)
	userDB = UserDB.UserDB()
	try:
		user = userDB.addUser(uid, passwd, email) 
	except UserDB.AlreadyExistUser:
		request.msg = 'Already exist user.'
		return do_view(request)
	request.sess['uid'] = user.uid
	return redirect(request.actionPath)

def do_login(request): 
	userDB = UserDB.UserDB()
	request.uid = request.uid.strip()
	if not request.uid:
		request.msg = 'Check your ID'
		return do_view(request)
	request.passwd = request.passwd.strip()
	if not request.passwd:
		request.msg = 'Check your password'
		return do_view(request)
	try:
		user = userDB.getUser(request.uid, request.passwd) # FIXME getUser를 써야하나? 그냥 User에서 가져오면 안되나 User.User(request.uid)
	except User.NotExistUser:
		request.msg = 'Check your ID'
		return do_view(request)
	except UserDB.NotCorrectPassword:
		request.msg = 'Check your password'
		return do_view(request)
	request.sess['uid'] = user.uid
	return redirect(request.actionPath)

def do_logout(request):
	request.sess.remove()
	request.sess['uid'] = 'anonymous'
	request.user = User.User(request.sess['uid']) 
	return redirect(request.actionPath)

def do_updateUser(request):
	# FIXME vvvv join이랑 중복
	passwd = request.passwd.strip()
	passwdConfirm = request.passwdConfirm.strip()
	email = request.email.strip()
	# uid 검사
	if request.user.uid.lower() == 'anonymous':
		request.msg = 'Invalid user ID'
		return do_view(request)
	# passwd 검사 # FIXME vvvv join이랑 중복
	if not (5 < len(passwd) < 17) or passwd != passwdConfirm:
		request.msg = 'Check your password (5 < 비밀번호 길이 < 17, password, passwordConfirm 이어야 합니다.)'
		return do_view(request)
	# email검사
	if email.count('@') != 1:
		request.msg = 'Check your email address'
		return do_view(request)
	# FIXME ^^^^^^^^^^^
	userDB = UserDB.UserDB()
	user = User.User(request.user.uid) # FIXME NotExistUser, InvalidUserData
	user.update(passwd, email)
	request.msg = 'Updated'
	return do_view(request)

def do_pageListInTag(request):
	title = "Page list in '%s' tag" % decode(request.tagName)
	tagObj = Tag.Tag(decode(request.tagName)) 
	return Response.Response('pageListInTag.teul', globals(), locals())

def do_info(request):
	timeStampList = Revision.getRevTimeStampList(request.categoryPath, request.pageName)
	timeStampList.reverse()
	if timeStampList:
		category = Category.getCategoryByCategoryPath(request.categoryPath)
		rev = Revision.Revision(category, request.pageName, timeStampList[0])  # FIXME 이 현재 권한 얻어오는게 따로 빠져야하는게 아닌가.
		if not rev.permCheck(request.user, 'read'): 
			return warning('Permission Error', "You don't have permission.")
	if not request.showAll and len(timeStampList) > 10:
		timeStampList = timeStampList[:10]
	return Response.Response('pageInfo.teul', globals(), locals())

def do_editCategoryForm(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	title = 'Category Edit : %s' % (category.categoryPath and category.categoryPath or 'Root')
	memberRead, memberWrite, groupRead, groupWrite, anonyRead, anonyWrite = category.perms.toBoolList() # skin에서 사용
	return Response.Response('editCategory.teul', globals(), locals())

def do_editCategory(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	category.editAttr(perms, request.group.split())
	request.msg = 'changed'
	return do_editCategoryForm(request)

def dox_upload(request):
	try:
		fileName = request.form['upload'].filename
		fileValue = request.form['upload'].value
	except: 
		fileName = ''
		fileValue = ''
	if not (fileName and fileValue):
		return warning('Upload Error', 'upload error')
	filePath = os.path.join(Repository.getAttachmentRepository(), fileName)
	file(filePath, 'wb').write(fileValue)
	return redirect(self.actionPath)
